@@ -48,10 +48,10 @@ import Data.Data.Lens (biplate)
 import Data.Either (rights)
 import Data.Monoid ((<>), mempty)
 import Data.Time.Clock.POSIX (getPOSIXTime)
-import Network.HTTP.Client (httpLbs, newManager, parseUrl, Response(..), setQueryString)
+import Network.HTTP.Client (httpLbs, newManager, parseUrlThrow, Response(..), setQueryString)
 import Network.HTTP.Client.TLS (tlsManagerSettings)
 import Network.HTTP.Types.Status (ok200)
-import Text.PrettyPrint.Free (pretty)
+import Data.Text.Prettyprint.Doc (pretty)
 
 data FetchValidationMethod = MatchPrimaryKeyFingerprint | MatchPrimaryOrAnySubkeyFingerprint | AnySelfSigned
     deriving (Bounded, Enum, Eq, Read, Show)
@@ -60,7 +60,7 @@ fetchKeys :: String -> FetchValidationMethod -> TwentyOctetFingerprint -> Except
 fetchKeys ks fvm q = do
     manager <- liftIO $ newManager tlsManagerSettings
 
-    request <- liftIO $ parseUrl (ks <> basereq)
+    request <- liftIO $ parseUrlThrow (ks <> basereq)
     let newreq = setQueryString (newqs q) request
     response <- liftIO $ httpLbs newreq manager
 
@@ -82,11 +82,11 @@ validateKeys :: BL.ByteString -> ExceptT String IO [(TK,TK)] -- FIXME: conduit f
 validateKeys larmors = do
     bytestrings <- ExceptT $ return $ fmap (mconcat . map armorToBS) (AA.decodeLazy larmors)
     keys <- liftIO . runConduitRes $ CB.sourceLbs bytestrings .| conduitGet get .| conduitToTKsDropping .| CL.consume
-    cpt <- liftIO $ getPOSIXTime
+    cpt <- liftIO getPOSIXTime
     return . rights $ map (uncurry (liftA2 (,)) . (pure &&& processTK (Just cpt))) keys
     where
        armorToBS (Armor ArmorPublicKeyBlock _ bs) = bs
        armorToBS _ = mempty
 
 rearmorKeys :: [TK] -> B.ByteString
-rearmorKeys keys = if (null keys) then mempty else (AA.encode . return . Armor ArmorPublicKeyBlock [("Comment", "filtered by hokey")] . runPut . put . Block $ keys)
+rearmorKeys keys = if null keys then mempty else AA.encode . return . Armor ArmorPublicKeyBlock [("Comment", "filtered by hokey")] . runPut . put . Block $ keys
