@@ -1,5 +1,5 @@
 -- TKUtils.hs: hOpenPGP-tools TK-related common functions
--- Copyright © 2013-2016  Clint Adams
+-- Copyright © 2013-2018  Clint Adams
 --
 -- vim: softtabstop=4:shiftwidth=4:expandtab
 --
@@ -20,14 +20,14 @@ module HOpenPGP.Tools.TKUtils (
     processTK
 ) where
 
-import Codec.Encryption.OpenPGP.Fingerprint (fingerprint, eightOctetKeyID)
+import Codec.Encryption.OpenPGP.Fingerprint (eightOctetKeyID)
 import Codec.Encryption.OpenPGP.Signatures (verifyTKWith, verifySigWith, verifyAgainstKeys)
 import Codec.Encryption.OpenPGP.Types
 import Control.Error.Util (hush)
 import Control.Arrow (second)
-import Control.Lens ((^.), (&), _1, _2, mapped, over)
+import Control.Lens ((^.), _1)
 import Data.List (sortBy)
-import Data.Maybe (fromMaybe, mapMaybe, listToMaybe)
+import Data.Maybe (mapMaybe, listToMaybe)
 import Data.Ord (comparing, Down(..))
 import Data.Time.Clock.POSIX (posixSecondsToUTCTime, POSIXTime)
 
@@ -50,7 +50,7 @@ processTK mpt key = verifyTKWith (verifySigWith (verifyAgainstKeys [key])) (fmap
         isCT (SigSubPacket _ (SigCreationTime _)) = True
         isCT _ = False
         sigissuer (SigVOther 2 _) = Nothing
-        sigissuer (SigV3 {}) = Nothing
+        sigissuer SigV3{} = Nothing
         sigissuer (SigV4 _ _ _ ys xs _ _) = listToMaybe . mapMaybe (getIssuer . _sspPayload) $ (ys++xs) -- FIXME: what should this be if there are multiple matches?
         sigissuer (SigVOther _ _) = error "We're in the future." -- FIXME
         eoki pkp
