@@ -160,10 +160,10 @@ data PathsOutputFormat = Unstructured | JSON | YAML
 
 listO :: String -> Parser Options
 listO homedir = Options
-    <$> (fromMaybe (homedir ++ "/.gnupg/pubring.gpg") <$> optional (strOption
+    <$> strOption
         ( long "keyring"
        <> metavar "FILE"
-       <> help "file containing keyring" )))
+       <> help "file containing keyring" )
     <*> pure GraphViz -- unused
     <*> option auto
         ( long "output-format"
@@ -180,10 +180,10 @@ listO homedir = Options
 
 graphO :: String -> Parser Options
 graphO homedir = Options
-    <$> (fromMaybe (homedir ++ "/.gnupg/pubring.gpg") <$> optional (strOption
+    <$> strOption
         ( long "keyring"
        <> metavar "FILE"
-       <> help "file containing keyring" )))
+       <> help "file containing keyring" )
     <*> option auto
         ( long "output-format"
        <> metavar "FORMAT"
@@ -202,10 +202,10 @@ graphO homedir = Options
 
 findPathsO :: String -> Parser Options
 findPathsO homedir = Options
-    <$> (fromMaybe (homedir ++ "/.gnupg/pubring.gpg") <$> optional (strOption
+    <$> strOption
         ( long "keyring"
        <> metavar "FILE"
-       <> help "file containing keyring" )))
+       <> help "file containing keyring" )
     <*> pure GraphViz -- unused
     <*> option auto
         ( long "output-format"
