@@ -58,7 +58,11 @@ import Data.Maybe (fromMaybe, mapMaybe, listToMaybe)
 #if !MIN_VERSION_base(4,8,0)
 import Data.Monoid (mconcat, Monoid(..))
 #endif
+#if !MIN_VERSION_base(4,9,0)
 import Data.Monoid ((<>))
+#else
+import Data.Semigroup ((<>), Semigroup)
+#endif
 import Data.Ord (comparing, Down(..))
 import qualified Data.Set as Set
 import Data.Text (Text)
@@ -75,9 +79,14 @@ import Options.Applicative.Types (Parser)
 import System.IO (Handle, hFlush, hPutStrLn, stderr, stdin, stdout, hSetBuffering, BufferMode(..))
 import Data.Time.Locale.Compat (defaultTimeLocale)
 
-import qualified Text.PrettyPrint.ANSI.Leijen as PPAL
-import Text.PrettyPrint.Free (colon, indent, linebreak, list, pretty, text, (<+>), Doc)
-import System.Console.Terminfo.PrettyPrint (green, red, yellow, displayDoc, Effect)
+import Data.Text.Prettyprint.Doc (annotate, colon, flatAlt, hardline, indent, line, list, pretty, (<+>), Doc)
+import qualified Data.Text.Prettyprint.Doc.Render.Terminal as PPA
+import Data.Text.Prettyprint.Convert.AnsiWlPprint (toAnsiWlPprint)
+
+linebreak = flatAlt line mempty
+green = annotate (PPA.color PPA.Green)
+yellow = annotate (PPA.color PPA.Yellow)
+red = annotate (PPA.color PPA.Red)
 
 data KAS = KAS {
     pubkeyalgo :: Colored PubKeyAlgorithm
@@ -148,9 +157,18 @@ instance A.ToJSON b => A.ToJSON (FakeMap Text b) where
     toJSON = A.toJSON . Map.fromList . unFakeMap
 instance A.ToJSON RevocationStatus
 
+#if MIN_VERSION_base(4,9,0)
+instance Semigroup UIDReport where
+    (<>) (UIDReport a b c d e) (UIDReport a' b' c' d' e') = UIDReport (a <> a') (b <> b') (c <> c') (d <> d') (e <> e')
+#endif
+
 instance Monoid UIDReport where
     mempty = UIDReport [] [] [] [] []
+#if !MIN_VERSION_base(4,9,0)
     mappend (UIDReport a b c d e) (UIDReport a' b' c' d' e') = UIDReport (a <> a') (b <> b') (c <> c') (d <> d') (e <> e')
+#else
+    mappend = (<>)
+#endif
 
 checkKey :: Maybe POSIXTime -> TK -> KeyReport
 checkKey mpt key = (\x -> x { keyBestOf = populateBestOf x, keyHasEncryptionCapableSubkey = hasEncryptionCapableSubkey (concatMap skUsageFlags (keySubkeys x)) }) KeyReport {
@@ -222,16 +240,16 @@ checkKey mpt key = (\x -> x { keyBestOf = populateBestOf x, keyHasEncryptionCapa
         decolorize (Colored _ _ x) = x
         colorizeUID u
             | '(' `elem` T.unpack u = Colored (Just Yellow) (Just "parenthesis in uid")  -- FIXME: be more discerning
-            | not ('<' `elem` T.unpack u) = Colored (Just Yellow) (Just "no left angle bracket in uid")  -- FIXME: be more discerning
+            | '<' `notElem` T.unpack u = Colored (Just Yellow) (Just "no left angle bracket in uid")  -- FIXME: be more discerning
             | otherwise = Colored Nothing Nothing
         findRevocationReason = concatMap grabReasons . filter isCertRevocationSig
         grabReasons (SigV4 CertRevocationSig _ _ has _ _ _) = mapMaybe (grabReasons' . _sspPayload) has
         grabReasons _ = []
         grabReasons' (ReasonForRevocation a b) = Just (RevocationStatus True (show a) b)
         grabReasons' _ = Nothing
-        kufs s = map (colorizeKUFs s . (\(SigSubPacket _ (KeyFlags x)) -> x) . maybe undefined id . find isKUF . hasheds) . newestWith (any isKUF . hasheds)
+        kufs s = map (colorizeKUFs s . (\(SigSubPacket _ (KeyFlags x)) -> x) . fromMaybe undefined . find isKUF . hasheds) . newestWith (any isKUF . hasheds)
         colorizeKUFs False x = uncurry Colored (if (Set.member EncryptStorageKey x || Set.member EncryptCommunicationsKey x) && (Set.member SignDataKey x || Set.member CertifyKeysKey x) then (Just Yellow, Just "both signing & encryption") else (Just Green, Nothing)) x
-        colorizeKUFs True x = uncurry Colored (if (Set.member CertifyKeysKey x) then (Just Red, Just "certification-capable subkey") else (if (Set.member EncryptStorageKey x || Set.member EncryptCommunicationsKey x) && Set.member SignDataKey x then (Just Yellow, Just "both signing & encryption") else (Just Green, Nothing))) x
+        colorizeKUFs True x = uncurry Colored (if Set.member CertifyKeysKey x then (Just Red, Just "certification-capable subkey") else (if (Set.member EncryptStorageKey x || Set.member EncryptCommunicationsKey x) && Set.member SignDataKey x then (Just Yellow, Just "both signing & encryption") else (Just Green, Nothing))) x
         newestWith p = take 1 . sortBy (comparing (Down . take 1 . sigcts)) . filter p -- FIXME: this is terrible
         hasheds (SigV4 _ _ _ xs _ _ _) = xs
         hasheds _ = []
@@ -265,19 +283,19 @@ checkKey mpt key = (\x -> x { keyBestOf = populateBestOf x, keyHasEncryptionCapa
                                 (True, _, _) -> Colored Nothing Nothing False
         colorizeF pf fp = uncurry Colored (if pf == fp then (Just Red, Just "subkey has same fingerprint as primary key") else (Just Green, Nothing)) fp
 
-prettyKeyReport :: POSIXTime -> TK -> Doc Effect
+prettyKeyReport :: POSIXTime -> TK -> Doc PPA.AnsiStyle
 prettyKeyReport cpt key = do
  let keyReport = checkKey (Just cpt) key
  execWriter $ tell
-   ( linebreak <> text "Key has potential validity" <> colon <+> text (keyStatus keyReport)
-  <> linebreak <> text "Key has fingerprint" <> colon <+> pretty (SpacedFingerprint (keyFingerprint keyReport))
-  <> linebreak <> text "Checking to see if key is OpenPGPv4" <> colon <+> coloredToColor (text . show) (keyVer keyReport)
-  <> linebreak <> (\kas -> text "Checking to see if key is RSA or DSA (>= 2048-bit)" <> colon <+> coloredToColor pretty (pubkeyalgo kas) <+> coloredToColor (maybe (text "unknown") pretty) (pubkeysize kas)) (keyAlgorithmAndSize keyReport)
-  <> linebreak <> text "Checking user-ID- and user-attribute-related items" <> colon
+   ( linebreak <> pretty "Key has potential validity" <> colon <+> pretty (keyStatus keyReport)
+  <> linebreak <> pretty "Key has fingerprint" <> colon <+> pretty (SpacedFingerprint (keyFingerprint keyReport))
+  <> linebreak <> pretty "Checking to see if key is OpenPGPv4" <> colon <+> coloredToColor (pretty . show) (keyVer keyReport)
+  <> linebreak <> (\kas -> pretty "Checking to see if key is RSA or DSA (>= 2048-bit)" <> colon <+> coloredToColor pretty (pubkeyalgo kas) <+> coloredToColor (maybe (pretty "unknown") pretty) (pubkeysize kas)) (keyAlgorithmAndSize keyReport)
+  <> linebreak <> pretty "Checking user-ID- and user-attribute-related items" <> colon
   <> mconcat (map (uidtrip (keyCreationTime keyReport) . gottabeabetterway) (unFakeMap (keyUIDsAndUAts keyReport)))
-  <> linebreak <> text "Checking subkeys" <> colon
-  <> linebreak <> indent 2 (text "one of the subkeys is encryption-capable" <> colon <+> coloredToColor pretty (keyHasEncryptionCapableSubkey keyReport))
-  <> mconcat (map (subkeyrep) (keySubkeys keyReport))
+  <> linebreak <> pretty "Checking subkeys" <> colon
+  <> linebreak <> indent 2 (pretty "one of the subkeys is encryption-capable" <> colon <+> coloredToColor pretty (keyHasEncryptionCapableSubkey keyReport))
+  <> mconcat (map subkeyrep (keySubkeys keyReport))
   <> linebreak
    )
     where
@@ -286,24 +304,24 @@ prettyKeyReport cpt key = do
         coloredToColor f (Colored (Just Red) _ x) = red (f x)
         coloredToColor f (Colored Nothing _ x) = f x
         uidtrip ts (u, ur)
-           | null (uidRevocationStatus ur) = linebreak <> indent 2 (coloredToColor text (fmap T.unpack u)) <> colon
-          <> linebreak <> indent 4 (text "Self-sig hash algorithms" <> colon <+> (list . map (coloredToColor pretty) . uidSelfSigHashAlgorithms) ur)
-          <> linebreak <> indent 4 (text "Preferred hash algorithms" <> colon <+> mconcat (map (coloredToColor pretty) (uidPreferredHashAlgorithms ur)))
-          <> linebreak <> indent 4 (text "Key expiration times" <> colon <+> mconcat (map (coloredToColor list . fmap (map (text . keyExp ts))) (uidKeyExpirationTimes ur)))
-          <> linebreak <> indent 4 (text "Key usage flags" <> colon <+> (list . map (coloredToColor (pretty . Set.toList))) (uidKeyUsageFlags ur))
-           | otherwise = linebreak <> indent 2 (coloredToColor text (fmap T.unpack u)) <> colon <+> text "[revoked]"
-          <> linebreak <> indent 4 (text "Revocation code" <> colon <+> list (map (text . revocationCode) (uidRevocationStatus ur)))
-          <> linebreak <> indent 4 (text "Revocation reason" <> colon <+> list (map (text . T.unpack . revocationReason) (uidRevocationStatus ur)))
+           | null (uidRevocationStatus ur) = linebreak <> indent 2 (coloredToColor pretty (fmap T.unpack u)) <> colon
+          <> linebreak <> indent 4 (pretty "Self-sig hash algorithms" <> colon <+> (list . map (coloredToColor pretty) . uidSelfSigHashAlgorithms) ur)
+          <> linebreak <> indent 4 (pretty "Preferred hash algorithms" <> colon <+> mconcat (map (coloredToColor pretty) (uidPreferredHashAlgorithms ur)))
+          <> linebreak <> indent 4 (pretty "Key expiration times" <> colon <+> mconcat (map (coloredToColor list . fmap (map (pretty . keyExp ts))) (uidKeyExpirationTimes ur)))
+          <> linebreak <> indent 4 (pretty "Key usage flags" <> colon <+> (list . map (coloredToColor (pretty . Set.toList))) (uidKeyUsageFlags ur))
+           | otherwise = linebreak <> indent 2 (coloredToColor pretty (fmap T.unpack u)) <> colon <+> pretty "[revoked]"
+          <> linebreak <> indent 4 (pretty "Revocation code" <> colon <+> list (map (pretty . revocationCode) (uidRevocationStatus ur)))
+          <> linebreak <> indent 4 (pretty "Revocation reason" <> colon <+> list (map (pretty . T.unpack . revocationReason) (uidRevocationStatus ur)))
         keyExp ts ke = (show . pretty) ke ++ " = " ++ formatTime defaultTimeLocale "%c" (posixSecondsToUTCTime (realToFrac ts + realToFrac ke))
         gottabeabetterway (a, Colored x y z) = (Colored x y a, z)
-        subkeyrep skr = linebreak <> indent 2 (text "fpr" <> colon <+> coloredToColor pretty (fmap SpacedFingerprint (skFingerprint skr)))
-                     <> linebreak <> indent 4 (text "version" <> colon <+> coloredToColor pretty (skVer skr))
-                     <> linebreak <> indent 4 (text "timestamp" <> colon <+> pretty (skCreationTime skr))
-                     <> linebreak <> indent 4 ((\kas -> text "algo/size" <> colon <+> coloredToColor pretty (pubkeyalgo kas) <+> coloredToColor (maybe (text "unknown") pretty) (pubkeysize kas)) (skAlgorithmAndSize skr))
-                     <> linebreak <> indent 4 (text "binding sig hash algorithms" <> colon <+> (list . map (coloredToColor pretty) . skBindingSigHashAlgorithms) skr)
-                     <> linebreak <> indent 4 (text "usage flags" <> colon <+> (list . map (coloredToColor (pretty . Set.toList))) (skUsageFlags skr))
-                     <> linebreak <> indent 4 (text "embedded cross-cert" <> colon <+> (coloredToColor pretty . ccPresent . skCrossCerts) skr)
-                     <> linebreak <> indent 4 (text "cross-cert hash algorithms" <> colon <+> (list . map (coloredToColor pretty) . ccHashAlgorithms . skCrossCerts) skr)
+        subkeyrep skr = linebreak <> indent 2 (pretty "fpr" <> colon <+> coloredToColor pretty (fmap SpacedFingerprint (skFingerprint skr)))
+                     <> linebreak <> indent 4 (pretty "version" <> colon <+> coloredToColor pretty (skVer skr))
+                     <> linebreak <> indent 4 (pretty "timestamp" <> colon <+> pretty (skCreationTime skr))
+                     <> linebreak <> indent 4 ((\kas -> pretty "algo/size" <> colon <+> coloredToColor pretty (pubkeyalgo kas) <+> coloredToColor (maybe (pretty "unknown") pretty) (pubkeysize kas)) (skAlgorithmAndSize skr))
+                     <> linebreak <> indent 4 (pretty "binding sig hash algorithms" <> colon <+> (list . map (coloredToColor pretty) . skBindingSigHashAlgorithms) skr)
+                     <> linebreak <> indent 4 (pretty "usage flags" <> colon <+> (list . map (coloredToColor (pretty . Set.toList))) (skUsageFlags skr))
+                     <> linebreak <> indent 4 (pretty "embedded cross-cert" <> colon <+> (coloredToColor pretty . ccPresent . skCrossCerts) skr)
+                     <> linebreak <> indent 4 (pretty "cross-cert hash algorithms" <> colon <+> (list . map (coloredToColor pretty) . ccHashAlgorithms . skCrossCerts) skr)
 
 jsonReport :: POSIXTime -> TK -> BL.ByteString
 jsonReport ps = A.encode . checkKey (Just ps)
@@ -336,7 +354,7 @@ lintO = LintOptions
        <> ofHelp
         )
     where
-        ofHelp = helpDoc . Just $ PPAL.text "output format" <> PPAL.hardline <> PPAL.list (map (PPAL.text . show) ofchoices)
+        ofHelp = helpDoc . Just . toAnsiWlPprint $ pretty "output format" <> hardline <> list (map (pretty . show) ofchoices)
         ofchoices = [minBound..maxBound] :: [LintOutputFormat]
 
 fetchO :: Parser FetchOptions
@@ -357,19 +375,23 @@ fetchO = FetchOptions
         )
     <*> argument (eitherReader strToFP) ( metavar "FINGERPRINT" )
     where
-        vmHelp = helpDoc . Just $ PPAL.text "validation method" <> PPAL.hardline <> PPAL.list (map (PPAL.text . show) vmchoices)
+        vmHelp = helpDoc . Just . toAnsiWlPprint $ pretty "validation method" <> hardline <> list (map (pretty . show) vmchoices)
         vmchoices = [minBound..maxBound] :: [FetchValidationMethod]
         strToFP = parseFingerprint . T.pack
 
 dispatch :: Command -> IO ()
 dispatch (CmdFetch o) = banner' stderr >> hFlush stderr >> doFetch o
 dispatch (CmdLint o) = banner' stderr >> hFlush stderr >> doLint o
-dispatch (CmdCanonicalize) = banner' stderr >> hFlush stderr >> doCanonicalize
+dispatch CmdCanonicalize = banner' stderr >> hFlush stderr >> doCanonicalize
 
 main :: IO ()
 main = do
     hSetBuffering stderr LineBuffering
-    customExecParser (prefs showHelpOnError) (info (helper <*> versioner "hokey" <*> cmd) (headerDoc (Just (banner "hokey")) <> progDesc "hOpenPGP Key utility" <> footerDoc (Just (warranty "hokey")))) >>= dispatch
+    customExecParser (prefs showHelpOnError)
+                     (info (helper <*> versioner "hokey" <*> cmd)
+                           (headerDoc (Just (toAnsiWlPprint (banner "hokey")))
+                         <> progDesc "hOpenPGP Key utility"
+                         <> footerDoc (Just (toAnsiWlPprint (warranty "hokey"))))) >>= dispatch
 
 cmd :: Parser Command
 cmd = hsubparser
@@ -384,13 +406,12 @@ doLint o = do
     keys <- runConduitRes $ CB.sourceHandle stdin .| conduitGet get .| conduitToTKsDropping .| CL.consume
     output (lintOutputFormat o) cpt keys
     where
-        output Pretty cpt = mapM_ (displayDoc 0.8 . prettyKeyReport cpt)
+        output Pretty cpt = mapM_ (PPA.putDoc . prettyKeyReport cpt)
         output JSON cpt = mapM_ (BL.putStr . flip BL.append (BL.singleton 0x0a) . jsonReport cpt)
         output YAML cpt = mapM_ (B.putStr . yamlReport cpt)
 
 doCanonicalize :: IO ()
-doCanonicalize = do
-    runConduitRes $ CB.sourceHandle stdin .| conduitGet get .|
+doCanonicalize = runConduitRes $ CB.sourceHandle stdin .| conduitGet get .|
                    conduitToTKsDropping .| CL.map canonicalize .|
                    CL.map put .| conduitPut .| CB.sinkHandle stdout
     where
@@ -410,14 +431,14 @@ doFetch o = do
         Right ks -> B.putStr $ rearmorKeys ks
 
 banner' :: Handle -> IO ()
-banner' h = PPAL.hPutDoc h (banner "hokey" <> PPAL.hardline <> warranty "hokey" <> PPAL.hardline)
+banner' h = PPA.hPutDoc h (banner "hokey" <> hardline <> warranty "hokey" <> hardline)
 
 sigissuer :: SignaturePayload -> Maybe EightOctetKeyId
 getIssuer :: SigSubPacketPayload -> Maybe EightOctetKeyId
 hashAlgo :: SignaturePayload -> HashAlgorithm
 
 sigissuer (SigVOther 2 _) = Nothing
-sigissuer (SigV3 {}) = Nothing
+sigissuer SigV3{} = Nothing
 sigissuer (SigV4 _ _ _ ys xs _ _) = listToMaybe . mapMaybe (getIssuer . _sspPayload) $ (ys++xs) -- FIXME: what should this be if there are multiple matches?
 sigissuer (SigVOther _ _) = error "We're in the future." -- FIXME
 
